﻿var Promise = require('promise'),
    util = require('util'),
    debuglog = util.debuglog ? util.debuglog('sl') : require('./alt-debug-log.js').debuglog('sl');

function DiffService(cfg, needle) {
    this.getPreviousMapping = function getPreviousMapping(appName, branch, build) {
        var promise = new Promise(function (resolve, reject) {
            var opts = {
                json: true,
                compressed: true
            };
            if (cfg.proxy) //http://user:pass@proxy.server.com:3128
                opts.proxy = cfg.proxy;
            
            var urlParts = [cfg.server, 'v1', 'builddiff', 'mapping', encodeURIComponent(cfg.customerId), encodeURIComponent(branch), encodeURIComponent(appName)];
            
            var url = urlParts.join('/') + '?currentBuild=' + encodeURIComponent(build);
            debuglog('Checking previous build: at ' + url);
            needle.get(url, opts, function (err, resp, body) {
                if (err) {
                    debuglog('Failed getting previous build: ' + err);
                    resolve(null);
                    //reject(err);
                    return;
                }
                else {
                    if (resp.statusCode == 404) {
                        resolve(null);
                    }
                    else if (resp.statusCode == 200) {
                        try {
                            var ret = resp.body.meta ? resp.body : JSON.parse(resp.body.toString());
                            debuglog('Got previous mapping from server: ' + JSON.stringify(ret));
                            resolve(ret);
                        } catch (e) {
                            debuglog('Failed getting a response from the server: ' + e);
                            resolve(null);
                        }
                    }
                    else resolve(null);
                }
            });
        });
        return promise;
    };
    
    this.findMappingForNewBranch = function findMappingForNewBranch(handlerParams, branchLog) {
        
        var _this = this;        

        var promise = new Promise(function (resolve, reject) {
            
            var appName = handlerParams.buildArguments.appname;
            var branch = handlerParams.buildArguments.branch;
            var build = handlerParams.buildArguments.build;

            if (!branchLog) {
                debuglog('Branch history was not found, automatic branch detection aborted');
                return resolve(null);
            }
            
            if (branchLog.length < 2) {
                debuglog('Branch history is too short (' + branchLog.length + '), automatic branch detection aborted');
                return resolve(null);
            }
            var opts = {
                json: true,
                compressed: true
            };
            if (cfg.proxy) //http://user:pass@proxy.server.com:3128
                opts.proxy = cfg.proxy;
            
            var urlParts = [cfg.server, 'v1', 'builddiff', 'mapping', 'query', encodeURIComponent(cfg.customerId), encodeURIComponent(appName)];
            
            var query = branchLog;
            
            var url = urlParts.join('/');
            needle.post(url, query, opts, function (err, resp, body) {
                if (err) {
                    debuglog('Failed creating query for automatic branch detection: ' + err);
                    resolve(null);
                    return;
                }
                else {
                    if (resp.statusCode == 404) {
                        resolve(null);
                    }
                    else if (resp.statusCode == 201) {
                        try {
                            var queryId = resp.body.queryId;
                            var interval = 1000;
                            var attempts = 10;
                            queryBranchDetection(appName, queryId, attempts, interval, function (err, mapping) { //10 attemps, 10 seconds
                                if (err) {
                                    reject(err);
                                }
                                else {
                                    resolve(mapping);
                                }
                            });
                            
                            //debuglog('Got previous mapping from server: ' + JSON.stringify(ret));
                            //resolve(ret);
                        } catch (e) {
                            debuglog('Failed getting a response from the server: ' + e);
                            resolve(null);
                        }
                    }
                    else resolve(null);
                }
            });
        });
        return promise;
    }
    
    function queryBranchDetection(appName, queryId, attemptsLeft, interval, callback) {
        if (attemptsLeft === 0) {
            debuglog("Failed to get response from server (automatic branch detection timeout).");
            callback("Failed to get response from server (automatic branch detection timeout).");
            return;
        }
        
        setTimeout(function () {
            var opts = {
                json: true,
                compressed: true
            };
            if (cfg.proxy) //http://user:pass@proxy.server.com:3128
                opts.proxy = cfg.proxy;
            
            var urlParts = [cfg.server, 'v1', 'builddiff', 'mapping', 'query', encodeURIComponent(cfg.customerId), encodeURIComponent(appName), queryId];
            
            var url = urlParts.join('/');// + '?queryId=' + queryId;
            needle.get(url, opts, function (err, resp, body) {
                if (err) {
                    debuglog('Failed query for automatic branch detection result: ' + err);
                    callback(err);
                    return;
                }
                else {
                    if (resp.statusCode == 404) {
                        callback(null);
                    }
                    if (resp.statusCode == 200) {
                        try {
                            var ret = resp.body;
                            if (ret.status === 'ready' && ret.mapping) {
                                callback(null, ret.mapping);
                            }
                            else if (ret.status == 'notfound') {
                                callback(null, null);
                            }
                            else {
                                //try again
                                queryBranchDetection(appName, queryId, attemptsLeft - 1, interval, callback);
                            }
                        } catch (e) {
                            debuglog('Failed getting a response from the server: ' + e);
                            //try again
                            queryBranchDetection(appName, queryId, attemptsLeft - 1, interval, callback);
                        }
                    }
                    else {
                        //try again
                        queryBranchDetection(appName, queryId, attemptsLeft - 1, interval, callback);
                    }
                }
            });

        }, interval);
    }
    
    this.submitMapping = function (mapping) {
        var promise = new Promise(function (resolve, reject) {
            var opts = {
                json: true,
                compressed: true
            };
            if (cfg.proxy) //http://user:pass@proxy.server.com:3128
                opts.proxy = cfg.proxy;
            
            var url = cfg.server + '/v1/builddiff';
            debuglog('Submitting build diff: ' + JSON.stringify(mapping, null, '\t'));
            console.log('Sending approx '+JSON.stringify(mapping, null, '\t').length +" bytes to the server");
            needle.post(url, mapping, opts, function (err, resp, body) {
                debuglog('Submitted ' + (err?'with error':'succesfully'));
                if (err) {
                    debuglog(JSON.stringify({ err: err, resp: resp, body: body }, null, '\t'));
                    reject(err);
                }
                else {
                    resolve(resp.body);
                }
            });
        });
        
        return promise;
    }
}

module.exports.DiffService = DiffService;