﻿var statuses = require('../statuses.js');
var AnonymousMethodsHandler = require('./anonymous-methods-handler.js').AnonymousMethodsHandler;
var NamedMethodsHandler = require('./named-methods-handler.js').NamedMethodsHandler;

function HandlersChain(oldMapping, newMapping) {
	this.handlers = [];
	this.handlers.push(new NamedMethodsHandler(oldMapping, newMapping));
	this.handlers.push(new AnonymousMethodsHandler(oldMapping, newMapping));
}

HandlersChain.prototype.tryStore = function (method, isOldMapping) {
	for (var i = 0; i < this.handlers.length; i++) {
		var handler = this.handlers[i];
		handler.tryStore(method, isOldMapping);
	};
}

HandlersChain.prototype.handle = function (newMapping) {
	for (var i = 0; i < this.handlers.length; i++) {
		var handler = this.handlers[i];
		handler.handle(newMapping);
	};
}


function diffOldAndNew(oldMapping, newMapping) {
	var handlersChain = new HandlersChain(oldMapping, newMapping);
	
	oldMapping.methods.forEach(function (m) {
		if (m.status != statuses.METHOD_DELETED) {
			var isOldMapping = true;
			handlersChain.tryStore(m, isOldMapping);
		}
	});
	
	newMapping.methods.forEach(function (m) {
		var isOldMapping = false;
		handlersChain.tryStore(m, isOldMapping);
	});
	
	handlersChain.handle(newMapping);

	return newMapping;
}

module.exports = diffOldAndNew;