﻿var Storage = require('./storage.js').Storage;
var statuses = require('../statuses.js');

function NamedMethodsHandler(oldMapping, newMapping) {
	this.oldMapping = oldMapping;
	this.newMapping = newMapping;
	this.storage = new Storage();
}

NamedMethodsHandler.prototype.tryStore = function (method, isOldMapping) {
	if (method.isAnonymous)
		return;
	
	var methodId = method.id;
	if (isOldMapping) {
		this.storage.old[methodId] = method;
	} else {
		this.storage.new[methodId] = method;
	}
}

NamedMethodsHandler.prototype.handle = function (newMapping) {
	var oldLookup = this.storage.old;
	var newLookup = this.storage.new;
	Object.keys(oldLookup).forEach(function (methodId) {
		var oldMatch = oldLookup[methodId];
		var newMatch = newLookup[methodId];
		
		var oldHash = oldMatch.hash;
		var oldId = oldMatch.id;
		
		if (newMatch) {
			//newMatch.id = oldId;
			//compare hashes            
			var newHash = newMatch.hash;
			if (oldHash != newHash) {
				//method changed
				newMatch.status = statuses.METHOD_CHANGED;
			}
			else if (oldMatch.position[0] !== newMatch.position[0]) {
				//method moved
				newMatch.status = statuses.METHOD_MOVED;
			}
			else {
				//method unchanged
				newMatch.status = statuses.METHOD_NOT_CHANGED;
			}
			
			//Since this method was found, remove it from the newLookup (it should contain only the added functions).
			delete newLookup[methodId];
		} else {
			//Method deleted. Add it to the new mapping so the server will know its deleted.
			newMapping.methods.push({ id: oldId, status: statuses.METHOD_DELETED });
		}
	});
	
	Object.keys(newLookup).forEach(function (methodId) {
		//No need to create ID - named methods contain a relevant ID.
		var newData = newLookup[methodId];
		newData.status = statuses.METHOD_ADDED;
	});
}

module.exports.NamedMethodsHandler = NamedMethodsHandler;